@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2022 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2026 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -18,19 +18,14 @@
      misrepresented as being the original software.
   3. This notice may not be removed or altered from any source distribution.
 */
-#include "../../SDL_internal.h"
+#include "SDL_internal.h"
 
 #include "SDL_androidvideo.h"
 
-extern void Android_InitKeyboard(void);
-extern int Android_OnKeyDown(int keycode);
-extern int Android_OnKeyUp(int keycode);
+extern void Android_OnKeyDown(int keycode);
+extern void Android_OnKeyUp(int keycode);
 
-extern SDL_bool Android_HasScreenKeyboardSupport(_THIS);
-extern SDL_bool Android_IsScreenKeyboardShown(_THIS, SDL_Window * window);
-
-extern void Android_StartTextInput(_THIS);
-extern void Android_StopTextInput(_THIS);
-extern void Android_SetTextInputRect(_THIS, SDL_Rect *rect);
-
-/* vi: set ts=4 sw=4 expandtab: */
+extern bool Android_HasScreenKeyboardSupport(SDL_VideoDevice *_this);
+extern void Android_ShowScreenKeyboard(SDL_VideoDevice *_this, SDL_Window *window, SDL_PropertiesID props);
+extern void Android_HideScreenKeyboard(SDL_VideoDevice *_this, SDL_Window *window);
+extern void Android_RestoreScreenKeyboard(SDL_VideoDevice *_this, SDL_Window *window);
