@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2025 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2026 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -18,7 +18,7 @@
      misrepresented as being the original software.
   3. This notice may not be removed or altered from any source distribution.
 */
-#include "../../SDL_internal.h"
+#include "SDL_internal.h"
 
 #if defined(SDL_VIDEO_DRIVER_UIKIT) && (defined(SDL_VIDEO_OPENGL_ES) || defined(SDL_VIDEO_OPENGL_ES2))
 
@@ -31,13 +31,13 @@
 #include "../../events/SDL_keyboard_c.h"
 #include "../../events/SDL_mouse_c.h"
 #include "../../power/uikit/SDL_syspower.h"
-#include "SDL_loadso.h"
+#include "../../SDL_hints_c.h"
 #include <dlfcn.h>
 
 @interface SDLEAGLContext : EAGLContext
 
-/* The OpenGL ES context owns a view / drawable. */
-@property (nonatomic, strong) SDL_uikitopenglview *sdlView;
+// The OpenGL ES context owns a view / drawable.
+@property(nonatomic, strong) SDL_uikitopenglview *sdlView;
 
 @end
 
@@ -52,7 +52,7 @@
 
 @end
 
-void *UIKit_GL_GetProcAddress(_THIS, const char *proc)
+SDL_FunctionPointer UIKit_GL_GetProcAddress(SDL_VideoDevice *_this, const char *proc)
 {
     /* Look through all SO's for the proc symbol.  Here's why:
      * -Looking for the path to the OpenGL Library seems not to work in the iOS Simulator.
@@ -61,12 +61,12 @@ void *UIKit_GL_GetProcAddress(_THIS, const char *proc)
 }
 
 /*
-  note that SDL_GL_DeleteContext makes it current without passing the window
+  note that SDL_GL_DestroyContext makes it current without passing the window
 */
-int UIKit_GL_MakeCurrent(_THIS, SDL_Window * window, SDL_GLContext context)
+bool UIKit_GL_MakeCurrent(SDL_VideoDevice *_this, SDL_Window *window, SDL_GLContext context)
 {
     @autoreleasepool {
-        SDLEAGLContext *eaglcontext = (__bridge SDLEAGLContext *) context;
+        SDLEAGLContext *eaglcontext = (__bridge SDLEAGLContext *)context;
 
         if (![EAGLContext setCurrentContext:eaglcontext]) {
             return SDL_SetError("Could not make EAGL context current");
@@ -77,45 +77,26 @@ int UIKit_GL_MakeCurrent(_THIS, SDL_Window * window, SDL_GLContext context)
         }
     }
 
-    return 0;
+    return true;
 }
 
-void UIKit_GL_GetDrawableSize(_THIS, SDL_Window * window, int * w, int * h)
-{
-    @autoreleasepool {
-        SDL_WindowData *data = (__bridge SDL_WindowData *)window->driverdata;
-        UIView *view = data.viewcontroller.view;
-        if ([view isKindOfClass:[SDL_uikitopenglview class]]) {
-            SDL_uikitopenglview *glview = (SDL_uikitopenglview *) view;
-            if (w) {
-                *w = glview.backingWidth;
-            }
-            if (h) {
-                *h = glview.backingHeight;
-            }
-        } else {
-            SDL_GetWindowSize(window, w, h);
-        }
-    }
-}
-
-int UIKit_GL_LoadLibrary(_THIS, const char *path)
+bool UIKit_GL_LoadLibrary(SDL_VideoDevice *_this, const char *path)
 {
     /* We shouldn't pass a path to this function, since we've already loaded the
      * library. */
     if (path != NULL) {
         return SDL_SetError("iOS GL Load Library just here for compatibility");
     }
-    return 0;
+    return true;
 }
 
-int UIKit_GL_SwapWindow(_THIS, SDL_Window * window)
+bool UIKit_GL_SwapWindow(SDL_VideoDevice *_this, SDL_Window *window)
 {
     @autoreleasepool {
-        SDLEAGLContext *context = (__bridge SDLEAGLContext *) SDL_GL_GetCurrentContext();
+        SDLEAGLContext *context = (__bridge SDLEAGLContext *)SDL_GL_GetCurrentContext();
 
 #ifdef SDL_POWER_UIKIT
-        /* Check once a frame to see if we should turn off the battery monitor. */
+        // Check once a frame to see if we should turn off the battery monitor.
         SDL_UIKit_UpdateBatteryMonitoring();
 #endif
 
@@ -125,15 +106,15 @@ int UIKit_GL_SwapWindow(_THIS, SDL_Window * window)
          * We don't pump events here because we don't want iOS application events
          * (low memory, terminate, etc.) to happen inside low level rendering. */
     }
-    return 0;
+    return true;
 }
 
-SDL_GLContext UIKit_GL_CreateContext(_THIS, SDL_Window * window)
+SDL_GLContext UIKit_GL_CreateContext(SDL_VideoDevice *_this, SDL_Window *window)
 {
     @autoreleasepool {
         SDLEAGLContext *context = nil;
         SDL_uikitopenglview *view;
-        SDL_WindowData *data = (__bridge SDL_WindowData *) window->driverdata;
+        SDL_UIKitWindowData *data = (__bridge SDL_UIKitWindowData *)window->internal;
         CGRect frame = UIKit_ComputeViewFrame(window, data.uiwindow.screen);
         EAGLSharegroup *sharegroup = nil;
         CGFloat scale = 1.0;
@@ -145,7 +126,7 @@ SDL_GLContext UIKit_GL_CreateContext(_THIS, SDL_Window * window)
          * versions. */
         EAGLRenderingAPI api = major;
 
-        /* iOS currently doesn't support GLES >3.0. */
+        // iOS currently doesn't support GLES >3.0.
         if (major > 3 || (major == 3 && minor > 0)) {
             SDL_SetError("OpenGL ES %d.%d context could not be created", major, minor);
             return NULL;
@@ -156,11 +137,11 @@ SDL_GLContext UIKit_GL_CreateContext(_THIS, SDL_Window * window)
         }
 
         if (_this->gl_config.share_with_current_context) {
-            EAGLContext *currContext = (__bridge EAGLContext *) SDL_GL_GetCurrentContext();
+            EAGLContext *currContext = (__bridge EAGLContext *)SDL_GL_GetCurrentContext();
             sharegroup = currContext.sharegroup;
         }
 
-        if (window->flags & SDL_WINDOW_ALLOW_HIGHDPI) {
+        if (window->flags & SDL_WINDOW_HIGH_PIXEL_DENSITY) {
             /* Set the scale to the natural scale factor of the screen - the
              * backing dimensions of the OpenGL view will match the pixel
              * dimensions of the screen rather than the dimensions in points. */
@@ -173,7 +154,13 @@ SDL_GLContext UIKit_GL_CreateContext(_THIS, SDL_Window * window)
             return NULL;
         }
 
-        /* construct our view, passing in SDL's OpenGL configuration data */
+        int srgb = _this->gl_config.framebuffer_srgb_capable;
+        const char *srgbhint = SDL_GetHint(SDL_HINT_OPENGL_FORCE_SRGB_FRAMEBUFFER);
+        if (srgbhint && *srgbhint) {
+            srgb = SDL_GetStringBoolean(srgbhint, false) ? 1 : 0;  // there is no "skip" here, since initWithFrame expects it, so we'll treat it as false.
+        }
+
+        // construct our view, passing in SDL's OpenGL configuration data
         view = [[SDL_uikitopenglview alloc] initWithFrame:frame
                                                     scale:scale
                                             retainBacking:_this->gl_config.retained_backing
@@ -183,7 +170,7 @@ SDL_GLContext UIKit_GL_CreateContext(_THIS, SDL_Window * window)
                                                     aBits:_this->gl_config.alpha_size
                                                 depthBits:_this->gl_config.depth_size
                                               stencilBits:_this->gl_config.stencil_size
-                                                     sRGB:_this->gl_config.framebuffer_srgb_capable
+                                                     sRGB:srgb
                                              multisamples:samples
                                                   context:context];
 
@@ -191,21 +178,26 @@ SDL_GLContext UIKit_GL_CreateContext(_THIS, SDL_Window * window)
             return NULL;
         }
 
-        /* The context owns the view / drawable. */
+        SDL_PropertiesID props = SDL_GetWindowProperties(window);
+        SDL_SetNumberProperty(props, SDL_PROP_WINDOW_UIKIT_OPENGL_FRAMEBUFFER_NUMBER, view.drawableFramebuffer);
+        SDL_SetNumberProperty(props, SDL_PROP_WINDOW_UIKIT_OPENGL_RENDERBUFFER_NUMBER, view.drawableRenderbuffer);
+        SDL_SetNumberProperty(props, SDL_PROP_WINDOW_UIKIT_OPENGL_RESOLVE_FRAMEBUFFER_NUMBER, view.msaaResolveFramebuffer);
+
+        // The context owns the view / drawable.
         context.sdlView = view;
 
-        if (UIKit_GL_MakeCurrent(_this, window, (__bridge SDL_GLContext) context) < 0) {
-            UIKit_GL_DeleteContext(_this, (SDL_GLContext) CFBridgingRetain(context));
+        if (!UIKit_GL_MakeCurrent(_this, window, (__bridge SDL_GLContext)context)) {
+            UIKit_GL_DestroyContext(_this, (SDL_GLContext)CFBridgingRetain(context));
             return NULL;
         }
 
-        /* We return a +1'd context. The window's driverdata owns the view (via
+        /* We return a +1'd context. The window's internal owns the view (via
          * MakeCurrent.) */
-        return (SDL_GLContext) CFBridgingRetain(context);
+        return (SDL_GLContext)CFBridgingRetain(context);
     }
 }
 
-void UIKit_GL_DeleteContext(_THIS, SDL_GLContext context)
+bool UIKit_GL_DestroyContext(SDL_VideoDevice *_this, SDL_GLContext context)
 {
     @autoreleasepool {
         /* The context was retained in SDL_GL_CreateContext, so we release it
@@ -213,6 +205,7 @@ void UIKit_GL_DeleteContext(_THIS, SDL_GLContext context)
          * context is deallocated. */
         CFRelease(context);
     }
+    return true;
 }
 
 void UIKit_GL_RestoreCurrentContext(void)
@@ -225,13 +218,11 @@ void UIKit_GL_RestoreCurrentContext(void)
          finished running its own code for the frame. If this isn't done, the
          app may crash or have other nasty symptoms when Dictation is used.
          */
-        EAGLContext *context = (__bridge EAGLContext *) SDL_GL_GetCurrentContext();
+        EAGLContext *context = (__bridge EAGLContext *)SDL_GL_GetCurrentContext();
         if (context != NULL && [EAGLContext currentContext] != context) {
             [EAGLContext setCurrentContext:context];
         }
     }
 }
 
-#endif /* SDL_VIDEO_DRIVER_UIKIT */
-
-/* vi: set ts=4 sw=4 expandtab: */
+#endif // SDL_VIDEO_DRIVER_UIKIT
